############################################################################
# Matt Rodriguez, LBNL
# See Copyright for copyright notice!
###########################################################################


import getopt, sys
from pyOGSI.stub import *
from exceptions import Exception

obj_paths = {
    'counter':'/ogsa/services/samples/counter/basic/CounterFactoryService',
    'google':'/ogsa/services/samples/google/GoogleSearchFactoryService',
    'weather':'/ogsa/services/samples/weather/WeatherFactoryService'  
    }

class ServiceIF:
    """kw = {'ssl':0,
          'namespace':'ns1',
          'host':None,
          'port':None,
          'tracefile':sys.stdout,
          #'url':(inst_path+' HTTP/1.0\r\n')
          'url': None}
          """
    def __init__(self):
        self._wsdl = None
        self._stub_instance = None
        self._host = None
        self._port = None
        self._path = None
        self._kw = None
        return self

    

    def getWsdl(self):
        if self._path == None or self._host == None or self._port == None:
            raise Exception, "path or host or port values not set, cannot retrieve wsdl doc"
        else:
            if self._path[-14:] == 'FactoryService':
                path = '/schema/core/factory/factory_service.wsdl'
            elif self._path[:self._path.rfind('/')] == obj_paths['counter']:
                path = '/schema/samples/counter/counter_service.wsdl'
            elif self._path[:self._path.rfind('/')] == obj_paths['google']:
                path = '/schema/samples/google/google_search_service.wsdl'
            elif self._path[:self._path.rfind('/')] == obj_paths['weather']:
                path =  '/schema/samples/weather/weather_service.wsdl'
            else:
                raise RuntimeError, 'bad path for wsdl doc'
            url = 'http://' + self._host + ':' + self._port + path
            self._ws = Wsdl(url)
        
    def generateInstance(self):
        print "in generateInstance"
        stg = StubGenerator()
        stubby = stg.generate_stub(self._ws)
        a = stubby(**self._kw)
        print "returned type: ", type(a)
        return a
    def setKw(self):
        self._kw = {'ssl': 0,
                    'namespace':'ns1',
                    'host': self._host,
                    'port': self._port,
                    'tracefile': sys.stdout,
                    'url': (self._path +' HTTP/1.0\r\n')
                    }
        
    def setWsdlParams(self, host, port, path):
        self._host = host
        self._port = port
        self._path = path
        self.setKw()
        
    def createService(self, serviceName):
        
        self.getWsdl()
        stub_instance = self.generateInstance()
        args_dict = stub_instance.get_args_dict('createService')
        stub_instance.add_soap_header('<ns1:invocationID id='+'"'+ serviceName +'"'+ ' xmlns:ns1="http://ogsa.globus.org/"/>')
        try:
            stub_instance.createService(**args_dict)
        except Exception, ex:
            pass
        self._serviceInstances.append(serviceName)

    def runService(self, argsTuple):
        raise NotImplementedError, 'Should be implemented by child class'

    def getServiceList(self):
        return self._servicesInstances
    
class clientCounterService(ServiceIF):
    def __init__(self):
        self._parent = ServiceIF.__init__(self)
        self._serviceInstances = []
        
    def runService(self, argsTuple):
        self.getWsdl()
        stub_instance = self.generateInstance()
        addArg = argsTuple[0]
        subArg = argsTuple[1]
        args_dict = stub_instance.get_args_dict('add')
        args_dict['value'] = addArg 
        stub_instance.add(**args_dict)
        args_dict['value'] = subArg
        stub_instance.subtract(**args_dict)
        value = stub_instance.getValue()
        return value

class googleService(ServiceIF):

    def __init__(self):
        self._parent = ServiceIF.__init__(self)
        self._serviceInstances = []

    def runService(self, argsTuple):

        """Should return what you want to post to the gui
        """
        self.getWsdl()
        stub_instance = self.generateInstance()
        queryArg = argsTuple[0]
        keyArg = argsTuple[1]
        mResultsArg = argsTuple[2]
        startArg = argsTuple[3]
        filterArg = argsTuple[4]
        safeSearchArg = argsTuple[5]
        args_dict = stub_instance.get_args_dict('doGoogleSearch')
        args_dict['q'] = queryArg
        args_dict['key'] = keyArg
        args_dict['maxResults'] = mResultsArg
        args_dict['start'] = startArg
        args_dict['filter'] = filterArg
        args_dict['safeSearch'] = safeSearchArg
        stub_instance.doGoogleSearch(**args_dict)
        
    
class weatherService(ServiceIF):

    def __init__(self):
        self._parent = ServiceIF.__init__(self)
        self._serviceInstances = []        

    def runService(self, argsTuple):
         """Should return what you want to post to the gui
        """
        self.getWsdl()
        stub_instance = self.generateInstance()
        zipCodeArg = argsTuple[0]
        args_dict = stub_instance.get_args_dict('setZipCode')
        try:
            stub_instance.setZipCode(**args_dict)
        except Exception, ex:
            print 'DEBUG NOT HANDLING xsd:float response'
        stub_instance.getTemp()
    
class pyAkentiService(ServiceIF):
    def __init__(self):
        self._parent = ServiceIF.__init__(self)
        self._serviceInstances = []        

    def createService(self, serviceName):
        """I'm not sure where the wsdl doc that describes how to create
        this service is. The other 3 services use the same wsdl doc(factory.wsdl) so they just call the ServiceIF.createService method
        """
        print "need to create pyAkentiService"
        pass
    def runService(self, argsTuple):
        """ Fill this in to run the pyAkenti Service
           Also add the right path to line 79 in pyOGSIgui.py :)
        """
        """Should return what you want to post to the gui
        """
        pass
    
    
