############################################################################
# Matt Rodriguez, LBNL
# See Copyright for copyright notice!
###########################################################################
#
# NOTE: Resizing the GUI after switching panels is a bad idea. As is I don't
# really switch panels so much as place a panel over the old panel. If you want
# the GUI bigger just change the size parameters in defaults.cfg 
#

from wxPython.wx import *
import ConfigParser
from pyOGSI.clientInterface import *
import time

mWidgetCodes = {
    "ID_EXIT" : 1002,
    "ID_COUNTER_B" : 1003,
    "ID_PYAKENTI_B" : 1004,
    "ID_GOOGLE_B": 1005,
    "ID_WEATHER_B": 1006,
    "ID_CREATE_SERVICE_B":1007,
    "ID_RUN_SERVICE_B":1008
}

g_config = None
g_control = None

def init_globals():
    global g_config
    if g_config == None:
        g_config = ConfigParser.ConfigParser()
        g_config.read("./defaults.cfg")
        
class MyApp(wxApp):
    def OnInit(self):
        global g_config
        if g_config == None:
            g_config = ConfigParser.ConfigParser()
            g_config.read("./defaults.cfg")

        frame = childFrame(NULL, -1, "pyOGSI Client")
        frame.Show(true)
        self.SetTopWindow(frame)
        return true

class childFrame(wxFrame):
    def __init__(self, parent, ID, title):
        global g_control
        SIZE_H = g_config.getint("Dimensions", "Size_H")
        SIZE_V = g_config.getint("Dimensions", "Size_V")
        wxFrame.__init__(self, None, ID, title,
                                 wxDefaultPosition, wxSize(SIZE_H, SIZE_V))

        sizer = wxBoxSizer(wxHORIZONTAL)
        self._gui = GuiPanel(self)
        g_control = Control(self._gui)
        sizer.Add(self._gui, 1, wxEXPAND)
        self.MakeMenu()
        self.SetAutoLayout(true)
        self.SetSizer(sizer)
        return
    
    def Quit(self, event):
        self.Close(true)

    def MakeMenu(self):
        fmenu = wxMenu()
        fmenu.Append(mWidgetCodes['ID_EXIT'], "E&xit", "Terminate the program")
        menuBar = wxMenuBar()
        menuBar.Append(fmenu,"&File")
        self.SetMenuBar(menuBar)
        EVT_MENU(self, mWidgetCodes['ID_EXIT'], self.Quit)

class Control:
    def __init__(self, gui):
        self._gui = gui
        self._state = None
        #TODO add the path for the pyAkentiService
        self._path_dict = {'WeatherService' : '/ogsa/services/samples/weather/WeatherFactoryService',
        'CounterService': '/ogsa/services/samples/counter/basic/CounterFactoryService',
        'GoogleService': '/ogsa/services/samples/google/GoogleSearchFactoryService',
        'PyAkentiService' : 'Dont know this path!!!'                   }
        
        self._serviceActions = None
        
    def switchServicePanel(self, newPanel):
        self._gui.removeServicePanel()
        self._gui.setServicePanel(newPanel)
        self._gui.guiSetup()
        self.setConfigPath()
        
    def setCounterServicePanel(self):
        newPanel = counterServicePanel(self._gui)
        self._state = 'CounterService'
        self._serviceActions = clientCounterService()
        self.switchServicePanel(newPanel)
        
    def setGoogleServicePanel(self):
        newPanel = googleServicePanel(self._gui)
        self._state = 'GoogleService'
        self._serviceActions = googleService()
        self.switchServicePanel(newPanel)
        
    def setWeatherServicePanel(self):
        newPanel = weatherServicePanel(self._gui)
        self._state = 'WeatherService'
        self._serviceActions = weatherService()
        self.switchServicePanel(newPanel)
        
    def setPyAkentiService(self):
        newPanel = pyAkentiServicePanel(self._gui)
        self._state = 'PyAkentiService'
        self._serviceActions = pyAkentiService()
        self.switchServicePanel(newPanel)
        
            
    def runService(self):
        host, port, path = self.getWsdlInfo()
        name = self.getCreatedServiceName()
        path +='/' + name
        self._serviceActions.setWsdlParams(host, port, path)
        sPanel = self.getServicePanel()
        argsT = sPanel.getArgs()
        value = self._serviceActions.runService(argsT)
        self.postOutput(value)

    def createService(self):
        host, port, path = self.getWsdlInfo()
        name = self.getServiceName()
        self._serviceActions.setWsdlParams(host, port, path)
        self._serviceActions.createService(name)
        self.appendServiceList(name)
        
    def postOutput(self, value):
        sPanel = self.getServicePanel()
        sPanel.postOutput(value)
        
    def getServicePanel(self):
        return self._gui.getServicePanel()

    def getServiceName(self):
        sPanel = self.getServicePanel()
        return sPanel.getServiceName()
    
    def getCreatedServiceName(self):    
        sPanel = self.getServicePanel()
        return sPanel.getCreatedServiceName()

    def appendServiceList(self, serviceName):
        sPanel = self.getServicePanel()
        sPanel.appendServiceName(serviceName)
        
    def getGuiPanel(self):
        return self._gui

    def getConfigPanel(self):
        return self._gui._topPanel

    def getWsdlInfo(self):
        cPanel = self.getConfigPanel()
        host = cPanel.getHost()
        port = cPanel.getPort()
        path = cPanel.getPath()
        return host,port, path

    def setConfigPath(self):
        cPanel = self.getConfigPanel()
        cPanel.setPath(self._path_dict[self._state])
        
class GuiPanel(wxPanel):
    def __init__(self,parent):
        global g_Control
        wxPanel.__init__(self,parent, -1)
        self._topPanel = configPanel(self)
        self._servicePanel = emptyPanel(self)
        self._outputPanel = outputPanel(self)
        self.guiSetup()
        return

    def guiSetup(self):
        """ Do initializations here"""
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._topPanel,1, wxEXPAND)
        servicePanelSizer = wxBoxSizer(wxVERTICAL)
        if self._servicePanel != None:
            servicePanelSizer.Add(self._servicePanel,1,wxEXPAND)

        sz.Add(servicePanelSizer,1, wxEXPAND)
        sz.Add(self._outputPanel,1, wxEXPAND)
        
        self.SetAutoLayout(true)
        self.SetSizer(sz)
        self._servicePanelSizer = servicePanelSizer
        self._mainSizer = sz
        self.Layout()
        
    def setServicePanel(self, servicePanel):
        self._servicePanel = servicePanel
        
    def getServicePanel(self):
        return self._servicePanel

    def removeServicePanel(self):
        self._mainSizer.Remove(self._servicePanelSizer)

        
class configPanel(wxPanel):
    
    def __init__(self, parent):
	wxPanel.__init__(self,parent, -1)
        self._button_panel = selectButtonPanel(self,{'CounterService': mWidgetCodes['ID_COUNTER_B'],'PyAkentiService':mWidgetCodes['ID_PYAKENTI_B'], 'GoogleService': mWidgetCodes['ID_GOOGLE_B'], 'WeatherService': mWidgetCodes['ID_WEATHER_B']})
	self._txt2 = wxStaticText(self, -1, '   Host    ')
        self._hostBox = wxComboBox(self,wxNewId())
        self._txt3 = wxStaticText(self, -1, '   Port    ')
	self._portBox = wxComboBox(self, wxNewId())
        self._txt4 =  wxStaticText(self, -1, '   Path    ')
        pathList = self.getPathList()
        self._pathBox = wxComboBox(self, wxNewId(), choices = pathList, style =wxCB_READONLY)
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._button_panel,1, wxEXPAND)
        sz1 = wxBoxSizer(wxHORIZONTAL)
	sz1.Add(self._txt2, 1, wxALIGN_CENTER)
        sz1.Add(self._hostBox, 3, wxEXPAND)
        sz1.Add(self._txt3, 1, wxALIGN_CENTER)
        sz1.Add(self._portBox, 2, wxEXPAND)
	sz1.Add(self._txt4, 1, wxALIGN_CENTER)
	sz1.Add(self._pathBox,8,wxEXPAND)

        sz.Add(sz1,1, wxEXPAND)
	self.SetSizer(sz)
        self.SetAutoLayout(true)
        defaultPanelColor(self)
        return 
    
    def getPort(self):
        return self._portBox.GetValue()

    def getHost(self):
        return self._hostBox.GetValue()

    def getPath(self):
        return self._pathBox.GetValue()

    def setPath(self, path):
        self._pathBox.SetValue(path)
        
    def getPathList(self):
        """Should ask the testClient for this """
        numPaths = g_config.getint("pathlist" , "numPaths")
        r = range(numPaths)
        pathList = []
        for x in r:
            name = g_config.get("pathlist", str(r[x]))
            pathList.append(name)

        return pathList

class ButtonPanel(wxPanel):
    def __init__(self,parent,buttons):
        wxPanel.__init__(self,parent, -1)
        self._buttons = buttons
        sz = wxBoxSizer(wxHORIZONTAL)
        sz1 = wxBoxSizer(wxHORIZONTAL)
        sz.Add(sz1,1, wxEXPAND)
        blist = buttons.items()
        count = 0
        blist.sort(lambda x,y: cmp(x[1],y[1]))
        for b in blist:
            btn = wxButton(self,b[1], b[0])
            EVT_BUTTON(self,b[1], self.onClick)
            sz.Add(btn,3, wxALIGN_CENTER)
            sz.Add(sz1,1, wxEXPAND)
            
        self.SetAutoLayout(1)
	self.SetSizer(sz)
	defaultPanelColor(self)
	return

    def onClick(self, event):
        raise NotImplementedError, 'Should be implemented by child class'

class selectButtonPanel(ButtonPanel):
    def __init__(self, parent, buttons):
        ButtonPanel.__init__(self, parent, buttons)

    def onClick(self, event):
        id = event.GetId()
        if id == self._buttons['CounterService']:
            g_control.setCounterServicePanel() 
        if id == self._buttons['PyAkentiService']:
            g_control.setPyAkentiService()
        if id == self._buttons['GoogleService']:
            g_control.setGoogleServicePanel()
        if id == self._buttons['WeatherService']:
            g_control.setWeatherServicePanel()

class createButtonPanel(ButtonPanel):
    def __init__(self, parent, buttons):
        ButtonPanel.__init__(self, parent, buttons)

    def onClick(self, event):
        id = event.GetId()
        if id == self._buttons['Create Service']:
            g_control.createService()

class runButtonPanel(ButtonPanel):
    def __init__(self, parent, buttons):
        ButtonPanel.__init__(self, parent, buttons)

    def onClick(self, event):
        print "HERE"
        id = event.GetId()
        if id == self._buttons['Run Service']:
            g_control.runService()
            
class emptyPanel(wxPanel):
    def __init__(self, parent):
        wxPanel.__init__(self, parent, wxNewId())
        defaultPanelColor(self)
        
class defaultServicePanel(wxPanel):
    def __init__(self, parent, id):
        wxPanel.__init__(self, parent, id)
        self._txt = wxStaticText(self, -1, '   Service to Create   ')
        self._txt2 = wxStaticText(self, -1, '   Services Created   ')
        self._txt3 = wxStaticText(self, -1, '   Service Reply   ')
        self._createButton = createButtonPanel(self,{'Create Service':mWidgetCodes['ID_CREATE_SERVICE_B']})
        self._runButton = runButtonPanel(self,{'Run Service': mWidgetCodes['ID_RUN_SERVICE_B']})
        self._serviceBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._serviceList = wxComboBox(self,wxNewId())
       
        self._valueBox = wxTextCtrl(self, wxNewId(), style = wxTE_READONLY)
        return self
        
    def guiSetup(self, extraSizer = None):
        sz = wxBoxSizer(wxHORIZONTAL)
        sz1 = wxBoxSizer(wxVERTICAL)
        sz1.Add(self._txt, 1, wxEXPAND)
        sz1.Add(self._createButton, 1, wxEXPAND)
        sz1.Add(self._serviceBox, 2, wxALIGN_CENTER)
        emptySizer = wxBoxSizer(wxHORIZONTAL)
        sz1.Add(emptySizer,5, wxEXPAND)
        
        sz2 = wxBoxSizer(wxVERTICAL)
        sz2.Add(self._txt2, 1, wxEXPAND)
        sz2.Add(self._runButton,1, wxEXPAND)
        sz2.Add(self._serviceList,5, wxEXPAND)

        sz3 = wxBoxSizer(wxVERTICAL)
        sz3.Add(self._txt3, 1, wxEXPAND)
        sz3.Add(self._valueBox,5,wxALIGN_CENTER)
        
        sz.Add(sz1,1, wxEXPAND)
        if extraSizer != None:
            sz.Add(extraSizer, 1, wxEXPAND)
            
        sz.Add(sz2,1, wxEXPAND)
        sz.Add(sz3,1, wxEXPAND)
        
        self.SetSizer(sz)
        self.SetAutoLayout(true)
        defaultPanelColor(self)

    def getServiceName(self):
        return self._serviceBox.GetValue()
        
    
    def getCreatedServiceName(self):
        return self._serviceList.GetStringSelection()
    
    def appendServiceName(self, serviceName):
        self._serviceList.Append(serviceName)
        self._serviceList.SetValue(serviceName)

    def postOutput(self, value):
        self._valueBox.SetValue(str(value))

class pyAkentiServicePanel(defaultServicePanel):
    def __init__(self, parent):
        self._id = wxNewId()
        self._parent = defaultServicePanel.__init__(self,parent, self._id)
        self._txt4 = wxStaticText(self, -1, 'Subject DN')
        self._txt5 = wxStaticText(self, -1, 'Issuer DN')
        self._txt6 = wxStaticText(self, -1, 'Resource')
        self._subjectBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._issuerBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._resourceBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self.setDefaultArgs()
        self.guiSetup()

    def guiSetup(self):
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._txt4, 1, wxEXPAND)
        sz.Add(self._subjectBox, 1, wxEXPAND)
        sz.Add(self._txt5, 1, wxEXPAND)
        sz.Add(self._issuerBox, 1, wxEXPAND)
        sz.Add(self._txt6, 1, wxEXPAND)
        sz.Add(self._resourceBox, 1, wxEXPAND)
        defaultServicePanel.guiSetup(self._parent,sz)

    def setDefaultArgs(self):
        subjectArg = g_config.get("pyAkentiService" ,"SubjectDN")
        issuerArg = g_config.get("pyAkentiService", "IssuerDN")
        resourceArg = g_config.get("pyAkentiService","Resource")
        self._subjectBox.SetValue(subjectArg)
        self._issuerBox.SetValue(issuerArg)
        self._resourceBox.SetValue(resourceArg)

    def getArgs(self):
        subjectArg = self._subjectBox.GetValue()
        issuerArg = self._issuerBox.GetValue()
        resourceArg = self._resourceBox.GetValue()
        return subjectArg, issuerArg, resourceArg
    
class counterServicePanel(defaultServicePanel):
    def __init__(self, parent):
        self._id = wxNewId()
        self._parent = defaultServicePanel.__init__(self,parent, self._id)
        self._txt4 = wxStaticText(self, -1, 'Value to Add')
        self._txt5 = wxStaticText(self, -1, 'Value to Subtract')
        self._addBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._subtractBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self.setDefaultArgs()
        self.guiSetup()    
    
    def guiSetup(self):
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._txt4,1, wxEXPAND)
        sz.Add(self._addBox, 1, wxEXPAND)
        sz.Add(self._txt5, 1, wxEXPAND)
        sz.Add(self._subtractBox,1, wxEXPAND)
        defaultServicePanel.guiSetup(self._parent,sz)

    def setDefaultArgs(self):
        addArg = g_config.get("CounterService", "add")
        subArg = g_config.get("CounterService", "subtract")
        self._addBox.SetValue(addArg)
        self._subtractBox.SetValue(subArg)
        
    def getArgs(self):
        addValue = self._addBox.GetValue()
        subValue = self._subtractBox.GetValue()
        return addValue, subValue
    
class googleServicePanel(defaultServicePanel):
    def __init__(self, parent):
        
        self._id = wxNewId()
        self._parent = defaultServicePanel.__init__(self,parent, self._id)
        self._txt4 = wxStaticText(self, -1, '  Query ')
        self._txt5 = wxStaticText(self, -1, '  Key ')
        self._txt6 = wxStaticText(self, -1, '  Max Results ')
        self._txt7 = wxStaticText(self, -1, '  Start ')
        self._txt8 = wxStaticText(self, -1, '  Filter ')
        self._txt9 = wxStaticText(self, -1, '  Safe Search ')

        self._queryBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._keyBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._maxResultsBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._startBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._filterBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self._safeSearchBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self.setDefaultArgs()
        self.guiSetup()
        
    def guiSetup(self):
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._txt4, 1, wxEXPAND)
        sz.Add(self._queryBox, 1, wxEXPAND)
        sz.Add(self._txt5, 1, wxEXPAND)
        sz.Add(self._keyBox, 1, wxEXPAND)
        sz.Add(self._txt6, 1, wxEXPAND)
        sz.Add(self._maxResultsBox, 1, wxEXPAND)
        sz.Add(self._txt7, 1, wxEXPAND)
        sz.Add(self._startBox,1, wxEXPAND) 
        sz.Add(self._txt8, 1, wxEXPAND)
        sz.Add(self._filterBox, 1, wxEXPAND)
        sz.Add(self._txt9, 1, wxEXPAND)
        sz.Add(self._safeSearchBox, 1, wxEXPAND)
        defaultServicePanel.guiSetup(self._parent, sz)
        
    def setDefaultArgs(self):
        queryArg = g_config.get("GoogleService", "query")
        keyArg = g_config.get("GoogleService", "key")
        mResultsArg = g_config.get("GoogleService", "maxResults")
        startArg = g_config.get("GoogleService", "start")
        filterArg = g_config.get("GoogleService", "filter")
        safeSearchArg = g_config.get("GoogleService", "safeSearch")
        self._queryBox.SetValue(queryArg)
        self._keyBox.SetValue(keyArg)
        self._maxResultsBox.SetValue(mResultsArg)
        self._startBox.SetValue(startArg)
        self._filterBox.SetValue(filterArg)
        self._safeSearchBox.SetValue(safeSearchArg)
        
    def getArgs(self):
        queryArg = self._queryBox.GetValue()
        keyArg = self._keyBox.GetValue()
        mResultsArg = self._maxResultsBox.GetValue()
        startArg = self._startBox.GetValue()
        filterArg = self._filterBox.GetValue()
        safeSearchArg = self._safeSearchBox.GetValue()
        return queryArg, keyArg, mResultsArg, startArg, filterArg, safeSearchArg
    
class weatherServicePanel(defaultServicePanel):

    def __init__(self, parent):
        self._id = wxNewId()
        self._parent = defaultServicePanel.__init__(self,parent, self._id)
        self._txt4 = wxStaticText(self, -1, '  Zip Code ')
        self._zipCodeBox = wxTextCtrl(self, self._id,style = wxTE_PROCESS_ENTER)
        self.setDefaultArgs()
        self.guiSetup()

    def guiSetup(self):
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._txt4, 1, wxEXPAND)
        sz.Add(self._zipCodeBox, 1, wxEXPAND)
        defaultServicePanel.guiSetup(self._parent, sz)

    def setDefaultArgs(self):
        zipArg = g_config.get("WeatherService", "zipCode")
        self._zipCodeBox.SetValue(zipArg)

    def getArgs(self):
        zipArg = self._zipCodeBox.GetValue()
        t = (zipArg,)
        return t
        
class outputPanel(wxPanel):
    def __init__(self, parent):
	wxPanel.__init__(self,parent, -1)
        self._outputPanel = wxTextCtrl(self,wxNewId(), style = wxTE_MULTILINE)
        self._title = wxStaticText(self, -1, ' Output  ')
        
        sz = wxBoxSizer(wxVERTICAL)
        sz1 = wxBoxSizer(wxHORIZONTAL)
        sz1.Add(self._title,1, wxALIGN_CENTER)
        sz.Add(sz1, 1, wxEXPAND)
        sz.Add(self._outputPanel,20, wxEXPAND)
        
        self.SetSizer(sz)
        self.SetAutoLayout(true)
        defaultPanelColor(self)
        
    def addText(self, newOutput):
        #Lose the old output for now
        self._outputPanel.SetValue(newOutput)
        

def defaultPanelColor( win ):
    """
    Set default color for panels
    """
    BGColor = getColor("BGColor")
    win.SetBackgroundColour(BGColor)
    FGColor = getColor("FGColor")
    win.SetForegroundColour(FGColor)
    return

def getColor(colorString):
    r = range(3)
    list = []
    for x in r:
        tmpString = colorString + str(r[x])
        colorNum = g_config.getint("Colors", tmpString)
        list.append(colorNum)

    color = wxColour(list[0], list[1], list[2])
    return color


def main():
    app = MyApp(0)
    app.MainLoop()



if __name__ == "__main__": main()
