############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import types, sys
from wsdllib import *
"""
try:
    from pyGlobus.ZSISOAP import GSIBinding as Binding
except Exception, ex:
    from ZSI.client import Binding
"""
from client import Binding_SOAPHEADER as Binding
from ZSI import TC, SoapWriter
from ZSI import _copyright, _children, _child_elements, \
        _inttypes, _stringtypes, _seqtypes, _find_arraytype, _find_href, \
        _find_type, \
        EvaluateException 
from ZSI.TCcompound import _check_typecode_list,_find_arrayoffset , \
     _find_arrayposition,_offset_pat,_position_pat


class indirectionStub:
    '''
    Protects this scenario
    x = a.add
    y = a.subtract
    x(10)
    '''
    op_requests = None
    tc_choice   = None
    def __init__(self, **kw):
        self.kw = kw
        self.tc = self.__class__.tc_choice()
        self.soap_header = None
        return
    def __str__(self):
        return (str(self.__class__) + ' op_requests: ' + str(self.__class__.op_requests))
    def __getattr__(self,cmd):
        print '__getattr__  cmd: %s' %(cmd)
        if cmd[:2] != '__' and cmd[-2:] != '__':
            return self.__Message(self.Call,cmd,self.__class__.op_requests)
    def get_args_dict(self,cmd):
        try:
            return self.__class__.op_requests.get_args_dict(cmd)
        except KeyError, ex:
            print "key: %s\nargs_dict: %s" %(cmd,self.__class__.op_requests.__str__())
            raise
    def add_soap_header(self, header):
        self.soap_header = header
        return
    def Call(self, cmd, *args, **args_dict):
        print 'Stub.Call cmd: %s %s' %(cmd, args_dict)
        tcs = self.__class__.op_requests.get_request(cmd,**args_dict)
        print 'tcs: ', tcs
        client = Binding(**self.kw)
        if self.soap_header:
            print "HEADER: ", self.soap_header
            client.AddSOAPHeader(self.soap_header)
        result_list = client.RPC(None,
                                 cmd,
                                 tcs,
                                 self.tc)
        print 'RETURN: "%s"'%result_list
        return result_list
    class __Message:
        def __init__(self, call, cmd, *l, **args_dict):
          print 'l ', l
          print 'args ', args_dict
          self.__call = call
          self.__cmd  = cmd
          return
        def __call__(self, **args_dict):
          print 'rpcall.__call__', args_dict
          return self.__call(self.__cmd, **args_dict)

      
class InterpreterInterface:
    def create_request(self, ws, type_in):
        raise NonImplementedError, 'this method must be implementated'
    def get_request(self):
        raise NonImplementedError, 'this method must be implementated'

    
class InterpreterAdapter(InterpreterInterface):
    def __init__(self, operations):
        self.__opRequests = operations()
        return
    def create_request(self, ws, type_in):
        self.__opRequests.create_request(ws,type_in)
        return self.__opRequests
    def create_request_from_msg(self, ws, msg_in):
        self.__opRequests.create_request_from_msg(ws,msg_in)
        return self.__opRequests

    
class RequestOperations(InterpreterInterface):
    def __init__(self):
        self.__seq_class_dict = {}
        self.__cmd_arg_dict = {}
        return
    def __str__(self):
        return str(self.__cmd_arg_dict)
    def __recurse(self, cmd, ws, e, varpath='/'):
        tcs = []
        tcd = {}
        try:
            begIdx = e.type.find(':')
        except AttributeError,ex:
            print 'DEBUG Cant handle: ', e
            return tcs,tcd
        if not e.complex:
            #Check for Primative Types, ogsa-types, or bomb
            if e.type[:begIdx] == 'xsd' or e.type == 'string' or e.type == 'dateTime':
                if e.type == 'xsd:int':
                    tcs = [TC_NOTYPE('ns1:'+e.name)]
                    tcd = {e.name:TC_NOTYPE}
                elif e.type == 'xsd:float':
                    tcs = [TC_NOTYPE('ns1:'+e.name)]
                    tcd = {e.name:TC_NOTYPE}
                elif e.type == 'xsd:string':
                    tcs = [TC_NOTYPE('ns1:'+e.name)]
                    tcd = {e.name:TC_NOTYPE}
                elif e.type == 'string':
                    tcs = [TC_NOTYPE('ns1:'+e.name)]
                    tcd = {e.name:TC_NOTYPE}
                elif e.type == 'dateTime':
                    tcs = [TC_NOTYPE('ns1:'+e.name)]
                    tcd = {e.name:TC_NOTYPE}
                else:
                    raise RuntimeError, ('UNKNOWN SIMPLE TYPE name(%s) type(%s)' %(e.name,e.type))
                
                self.__cmd_arg_dict[cmd].update({str(varpath+e.name+'/'):e.type})
            elif e.type[:begIdx] == 'ogsa-types' or e.type[:begIdx] == 'gsdl':
                this_type = e.type[begIdx+1:]
                try:
                    this_type = e.type[begIdx+1:]
                    ct_type = ws.types[this_type]
                except AttributeError,ex:
                    if this_type.sequence:
                        print 'DEBUG: self-contained element type'
                except KeyError,ex:
                    for i in ws.types.items(): print 'DEBUG: ',i
                    raise
                else:
                    if ct_type.complex:
                        print 'DEBUG: element type defined by %s' %(ct_type.name)
                    
                exec('class '+ct_type.name+'(Base): pass')
                CTclass = eval(ct_type.name)
                #CTclass.nsdict = 'xmlns:ns1="http://ogsa.gridforum.org/types"'
                CTclass.nsdict = 'xmlns:ns2="http://www.gridforum.org/namespaces/2002/10/gridServices"'
                varpath += ct_type.name
                for e1 in ct_type.sequence:
                    if e1.complex:
                        raise RuntimeError, 'DEBUG not handling complex here'
                    try:
                        if e1.type == 'string' and e1.name == 'primaryKey':
                            tcs1,tcd1 = self.__recurse(cmd,ws,e1,varpath)
                            tcs += tcs1
                            tcd.update(tcd1)
                        else:
                            print "DEBUG skip: %s %s" %(e1.name,e1.type)
                            continue
                    except AttributeError,ex:
                        print 'DEBUG Cant handle: ', e1
                tcs = [TC_Struct(CTclass,tcs,'ns2:'+e.name)]
                tcd = {e.name:(CTclass,tcd)}
                        
            else:
                raise RuntimeError, 'NOT READY YET type(%s)  name(%s)' %(e.type,e.name)        
        return tcs, tcd
    def get_args_dict(self, cmd):
        return self.__cmd_arg_dict[cmd]
    def create_request(self, ws, type_in):
        seq_class_dict = []
        self.__cmd_arg_dict[type_in.name] = {}
        if type_in.complex:
            raise RuntimeError, 'generate stub not handling this.'
        try:
            print '%s\n\t%s%s' %('>'*75,'type_in.name: ',type_in.name)
        except AttributeError, ex:
            print 'WARNING Skipping type_in(%s) lacks name' %(type_in)
            pass
        else:
            try:
                begIdx = type_in.type.find(':') + 1
                seq_type = ws.types[type_in.type[begIdx:]]
            except AttributeError, ex:
                seq_type = type_in                
            except KeyError, ex:
                seq_type = type_in
            else:
                if not seq_type.complex:
                    raise RuntimeError, 'Badly formatted wsdl, this type should be complex'
            
            print "DEBUG complex_type\n\tname %s\n\tseq %s" %(seq_type.name,seq_type.sequence)
            for e in seq_type.sequence:
                #Build typecode sequence & typecode dictionary
                tcs = []
                tcd = {}
                print 'DEBUG: %s %s' %(e.name,e.type)
                if not e.complex:
                    exec('class '+type_in.name+'Wrap(Base): pass')
                    CTwrapclass = eval(type_in.name+'Wrap')
                    #Need to grab namespace out of wsdl
                    CTwrapclass.nsdict = ''
                    if  e.type == 'ogsa-types:CreationType':
                        CTwrapclass.nsdict = 'xmlns="http://ogsa.gridforum.org/factory/factory_port_type"'
                    elif e.type == 'ogsa-types:ExtensibilityType':
                        pass
                    elif e.type[:11] == 'ogsa-types:':
                        pass
                    elif e.type[:4] == 'gsdl':
                        pass
                    else:
                        print "12DEBUG UNKOWN TYPE: ", e.type
                    tcs,tcd = self.__recurse(type_in.name,ws,e)
                else:
                    print "DEBUG UNKOWN TYPE: ", e.type
                try:
                    CTwrapclass.typecode=TC_Struct(CTwrapclass, tcs,\
                                                   type_in.name, inline=1)
                except UnboundLocalError, ex:
                    print "DEBUG OPERATION NOT BEING DEFINED"
                    self.__seq_class_dict[type_in.name] = []
                else:
                    self.__seq_class_dict[type_in.name] = [(CTwrapclass,tcd)]
        return

    def create_request_from_msg(self, ws, msg_in):
        #no part shall be an element
        self.__cmd_arg_dict[msg_in.name] = {}
        exec('class '+msg_in.name+'Wrap(Base): pass')
        CTwrapclass = eval(msg_in.name+'Wrap')
        tcs = []
        tcd = {}
        for n,part in msg_in.parts.items():
            if part.type == 'xsd:int':
                tcs += [TC_NOTYPE('ns1:'+n)]
                tcd.update({n:TC_NOTYPE})
            elif part.type == 'xsd:float':
                tcs += [TC_NOTYPE('ns1:'+n)]
                tcd.update({n:TC_NOTYPE})
            elif part.type == 'xsd:string':
                tcs += [TC_NOTYPE('ns1:'+n)]
                tcd.update({n:TC_NOTYPE})
            elif part.type == 'xsd:boolean':
                tcs += [TC_NOTYPE('ns1:'+n)]
                tcd.update({n:TC_NOTYPE})
            else:
                raise RuntimeError, ('from msg UNKNOWN SIMPLE TYPE name(%s) type(%s)' %(n,part.type))
                
        CTwrapclass.typecode=TC_Struct(CTwrapclass, tcs, msg_in.name, inline=1)
        self.__seq_class_dict[msg_in.name] = [(CTwrapclass,tcd)]
        return
    def get_request(self, cmd, **args_dict):
        print "get_request: ",self.__seq_class_dict
        try:
            tp = self.__seq_class_dict[cmd]
        except KeyError, ex:
            if not args_dict:
                xRequest.typecode=TC_Struct(xRequest,[],cmd,inline=1)
                tcs = xRequest()
            else:
                print 'DEBUG: ', args_dict
                for i in self.__seq_class_dict.items(): print 'ITEM: ',i
                raise RuntimeError, ('no __seq_class_dict[%s] but args??' %(cmd))
        else:
            print 'DEBUG\ttp(%i): %s' %(len(tp),tp)
            print 'args: ', args_dict
            if len(tp):
                if cmd == 'add' or cmd == 'subtract' or cmd == 'getValue' \
                   or cmd == 'doGoogleSearch' or cmd == 'setZipCode' or cmd == 'getTemp':
                    
                    CTwrapclass = tp[0][0]
                    tcs = CTwrapclass()
                    
                    for pkclass_key in tp[0][1].keys():
                        try:
                            tcs.__dict__[pkclass_key] = args_dict[pkclass_key]
                        except KeyError, ex:
                            print ex

                    print 'keys: ', tp[0][1].keys()
                    print 'pkclass_key: ', pkclass_key
                    print 'dict       : ', tp[0][1]
                else:
                    CTwrapclass = tp[0][0]
                    
                    CTclass_key = tp[0][1].keys()[0]
                    CTclass     = tp[0][1][CTclass_key][0]

                    try:
                        pkclass_key = tp[0][1][CTclass_key][1].keys()[0]
                    except IndexError, ex:
                        ct = CTclass()
                    else:
                        pkclass     = tp[0][1][CTclass_key][1][pkclass_key]
                        ct = CTclass()
                        ct.__dict__[pkclass_key] = args[0]
                        
                    tcs = CTwrapclass()
                    tcs.__dict__[CTclass_key] = ct
            else:
                raise RuntimeError, ('no class defs in __seq_class_dict[%s]??' %(cmd))
        return tcs
    
class StubGenerator:
    def __init__(self, adapter=InterpreterAdapter(RequestOperations)):
        self.adapter = adapter
        return
      
    def generate_stub(self, ws):
        '''
        generate new stub class for rpc, and
        typecode classes to handle return output.
        '''
        try:
            service = ws.services.values()[0]
        except IndexError, ex:
            raise RuntimeError, 'service undefined'

        #create stub class
        exec('class '+service.name+'(indirectionStub): pass')
        stub = eval(service.name)
        stub.wsdlobj = ws
        #create TC choice class
        tc_choice_class_name = 'TC_Choice_' + service.name
        exec('class '+tc_choice_class_name+'(TC_Base): pass')
        tc_choice = eval(tc_choice_class_name)

        for p in ws.services.values()[0].ports.values():
            #Get Binding
            begIdx = p.binding.find(':') + 1
            binding_tok = p.binding[begIdx:]
            binding = ws.bindings[binding_tok]
            #Get PortType
            pt_name = binding.type
            begIdx = pt_name.find(':') + 1
            pt_tok = pt_name[begIdx:]
            portType = ws.portTypes[pt_tok]

            for op, bop in binding.operations.items():
                #I/O msg names
                msg_name_in = portType.operations[op].input.message[4:]
                msg_name_out = portType.operations[op].output.message[4:]
                #I/O messages instances
                
                #print (('msg_name_in:  %s\nmessages: %s') %(msg_name_in,ws.messages))
                if msg_name_in.find('ns:')==0:
                    msg_in = ws.messages[msg_name_in[3:]]
                else:
                    msg_in = ws.messages[msg_name_in]
                    
                if msg_name_out.find('ns:')==0:
                    msg_out = ws.messages[msg_name_out[3:]]
                else:
                    msg_out = ws.messages[msg_name_out]
                    
                print ('msg_in.parts:  %s' %(msg_in.parts))    
                #I/O message Type names
                #msg_in_type = msg_in.parts['parameters'].element[4:]
                #msg_out_type = msg_out.parts['parameters'].element[4:]
      
                try:
                    part_in_type = msg_in.parts.values()[0].element
                except KeyError, ex:
                    stub.op_requests = self.adapter.create_request_from_msg(ws, msg_in)
                else:
                   idx = part_in_type.find(':')
                   if idx:
                       part_in_type = part_in_type[idx+1:]
                        
                   part_out_type = msg_out.parts.values()[0].element
                   idx = part_out_type.find(':')
                   if idx:
                       part_out_type = part_out_type[idx+1:]
 
                   #I/O message Type Instances
                   type_in = ws.types[part_in_type]
                   type_out = ws.types[part_out_type]
                   if  len(msg_in.parts.items()) == 1:
                       stub.op_requests = self.adapter.create_request(ws, type_in)
                   else:
                       raise RuntimeError, 'Error a message can only contain 1 element part'

                   
                #CREATE RESPONSE:  TC_Choice define type_out
                try:
                    if not type_out.complex:
                        try:
                            begIdx = type_out.type.find(':') + 1
                        except AttributeError, ex:
                            seq_dict = {}
                        else:
                            try:
                                complex_type = ws.types[type_out.type[begIdx:]]
                            except KeyError, ex:
                                for t in ws.types.items(): print t
                                raise
                            if not complex_type.complex:
                                raise RuntimeError, 'Badly formatted wsdl, complexType not complex'
                            seq_dict = {}
                            for e in complex_type.sequence:
                                seq_dict[str(e.name)] = e.type
                            
                        exec('class TC_'+type_out.name+'(TC_template): pass')
                        TC_class = eval('TC_'+type_out.name)
                    
                        exec('class D_'+type_out.name+'(Base): pass')
                        D_class = eval('D_'+type_out.name)
                    
                        tc2 = D_class()
                        print "seq: ", seq_dict
                        tc2.typecode = TC_class(type_out.name, **seq_dict)
                        tc_choice.name2class[type_out.name] = tc2
                    else:
                        raise RuntimeError, 'generate TC not handling this.'

                    stub.tc_choice = tc_choice
                except UnboundLocalError,ex:
                    pass
        return stub


class TC_Base(TC.TypeCode):
    name2class = {}
    def parse(self, elt, ps):
        return self.choice.parse(elt, ps)[1]
    
    def __getattr__(self, attr):
        if attr == 'choice':
            l = []
            for k,v in self.__class__.name2class.items():
                print v.typecode
                l += [v.typecode]
            choice = TC.Choice(l, 'Item')
            self.__dict__['choice'] = choice
            return choice
        raise AttributeError(attr)
    
class TC_template(TC.TypeCode):
    def __init__(self,name, **seq):
        TC.TypeCode.__init__(self)
        self.pname = name
        for n,v in seq.items():
            print "name of var: %s %s" %(n,v)
            if v == "xsd:int":
                self.tc = TC.Integer(n)
                #self.tc = TC.Integer('ns1:returnValue')
            else:
                print "*"*50+" DANGER " +50*"*"
                self.tc = TC.String(n)
                #raise RuntimeError, 'NOT READY YET'
    def parse(self, elt, ps):
        self.checkname(elt, ps)
        if self.nilled(elt, ps): return None
        p = self.tc.parse
        v = {}
        d = None
        for c in _child_elements(elt):
            d = p(c, ps)
        return d

class TC_NOTYPE(TC.String):
    def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
        objid = '%x' % id(pyobj)
        n = name or self.oname or ('E' + objid)
        print >>sw, '<%s>%s</%s>' % (n, pyobj, n)
        return

class Base: pass
class xRequest: pass
    
class TC_Struct(TC.TypeCode):
    def __init__(self, pyclass, ofwhat, pname=None, inorder=0, inline=0,
    mutable=1, hasextras=0, nsdict={}, **kw):
        '''pyclass -- the Python class to hold the fields
        ofwhat -- a list of fields to be in the struct
        hasextras -- ignore extra input fields
        inorder -- fields must be in exact order or not
        inline -- dont href/id when serializing
        mutable -- object could change between multiple serializations
        type -- the (URI,localname) of the datatype
        '''
        TC.TypeCode.__init__(self, pname, **kw)
        self.pyclass = pyclass
        self.inorder = inorder
        self.inline = inline
        self.mutable = mutable
        if self.mutable: self.inline = 1
        self.hasextras = hasextras
        self.type = kw.get('type')
        t = type(ofwhat)
        if t not in _seqtypes:
            raise TypeError(
                'Struct ofwhat must be list or sequence, not ' + str(t))
        self.ofwhat, self.lenofwhat = tuple(ofwhat), len(ofwhat)
        if kw.has_key('typed'):
            t = kw['typed']
            for w in self.ofwhat: w.typed = t
        if TC.TypeCode.typechecks:
            if self.pyclass != None and type(self.pyclass) != types.ClassType:
                raise TypeError('pyclass must be None or a class, not ' +
                        str(type(self.pyclass)))
            _check_typecode_list(self.ofwhat, 'Struct')
        self.nsdict = nsdict
    def parse(self, elt, ps):
        self.checkname(elt, ps)
        if self.type and \
        self.checktype(elt, ps) not in [ self.type, (None,None) ]:
            raise EvaluateException('Struct has wrong type', ps.Backtrace(elt))
        href = _find_href(elt)
        if href:
            if _children(elt):
                raise EvaluateException('Struct has content and HREF',
                        ps.Backtrace(elt))
            elt = ps.FindLocalHREF(href, elt)
        c = _child_elements(elt)
        count = len(c)
        if self.nilled(elt, ps): return None
        if count > self.lenofwhat and not self.hasextras:
            raise EvaluateException('Too many sub-elements', ps.Backtrace(elt))

        # Create the object.
        v = {}

        # Clone list of kids (we null it out as we process)
        c, crange = c[:], range(len(c))
        # Loop over all items we're expecting
        for i,what in [ (i, self.ofwhat[i]) for i in range(self.lenofwhat) ]:
            # Loop over all available kids
            for j,c_elt in [ (j, c[j]) for j in crange if c[j] ]:
                if what.name_match(c_elt):
                    # Parse value, and mark this one done.
                    try:
                        value = what.parse(c_elt, ps)
                    except EvaluateException, e:
                        e.str = '%s.%s: %s' % \
                                (self.pname or '?', what.aname or '?', e.str)
                        raise e
                    if what.repeatable:
                        if v.has_key(what.aname):
                            v[what.aname] = v[what.aname].append(value)
                        else:
                            v[what.aname] = [value]
                    else:
                        v[what.aname] = value
                    c[j] = None
                    break
                # No match; if it was supposed to be here, that's an error.
                if self.inorder and i == j:
                    raise EvaluateException('Out of order struct',
                            ps.Backtrace(c_elt))
            else:
                if not what.optional:
                    raise EvaluateException('Element "' + what.aname + \
                        '" missing from struct', ps.Backtrace(elt))
                if hasattr(what, 'default'):
                    v[what.aname] = what.default

        if not self.hasextras:
            extras = [c_elt for c_elt in c if c_elt]
            if len(extras):
                raise EvaluateException('Extra sub-elements (' + \
                        ','.join([c.nodeName for c in extras]) + ') found',
                        ps.Backtrace(elt))

        if not self.pyclass: return v

        try:
            pyobj = self.pyclass(self.aname)
        except Exception, e:
            raise TypeError("Constructing %s: %s" % (self.pname, str(e)))
        for key in v.keys():
            setattr(pyobj, key, v[key])
        return pyobj
    def serialize(self, sw, pyobj, inline=None, name=None, attrtext='', **kw):
        if inline or self.inline:
            self.cb(sw, pyobj, **kw)
        else:
            objid = '%x' % id(pyobj)
            n = name or self.oname or ('E' + objid)
            print >>sw, '<%s%s href="#%s"/>' % (n, attrtext, objid)
            sw.AddCallback(self.cb, pyobj)
    def cb(self, sw, pyobj, name=None, **kw):
        if not self.mutable and sw.Known(pyobj): return
        objid = '%x' % id(pyobj)
        n = name or self.oname or ('E' + objid)

        if self.inline:
            print >>sw, '<%s' % n     
        else:
            print >>sw, '<%s id="%s"' % (n, objid)

        #HACK NSDICT IN  sw.writeNSdict(nsdict)
        try:
            pyobj.nsdict
            print >>sw, ' %s>' % pyobj.nsdict    
        except AttributeError, ex:
            print "AttributeError ", ex
            print >>sw, '>'

        
        if self.pyclass:
            d = pyobj.__dict__
        else:
            d = pyobj
            if TC.TypeCode.typechecks and type(d) != types.DictType:
                raise TypeError("Classless struct didn't get dictionary")
        for what in self.ofwhat:
            print what.aname
            v = d.get(what.aname)
            if what.optional and v == None: continue
            try:
                if what.repeatable and type(v) in _seqtypes:
                    for v2 in v:
                        what.serialize(sw, v2)
                else:
                    what.serialize(sw, v)
            except Exception, e:
                print "DEBUG what: ", what
                raise RuntimeError('Serializing %s.%s, %s %s' %
                        (n, what.aname or '?', e.__class__.__name__, str(e)))
        print >>sw, '</%s>' % n

    """
    def cb(self, sw, pyobj, name=None, **kw):
        if not self.mutable and sw.Known(pyobj): return
        objid = '%x' % id(pyobj)
        n = name or self.oname or ('E' + objid)
        if self.inline:
            print >>sw, '<%s>' % n
        else:
            print >>sw, '<%s id="%s">' % (n, objid)
        if self.pyclass:
            d = pyobj.__dict__
        else:
            d = pyobj
            if TC.TypeCode.typechecks and type(d) != types.DictType:
                raise TypeError("Classless struct didn't get dictionary")
        for what in self.ofwhat:
            v = d.get(what.aname)
            if what.optional and v == None: continue
            try:
                if what.repeatable and type(v) in _seqtypes:
                    for v2 in v: what.serialize(sw, v2)
                else:
                    what.serialize(sw, v)
            except Exception, e:
                raise Exception('Serializing %s.%s, %s %s' %
                        (n, what.aname or '?', e.__class__.__name__, str(e)))
        print >>sw, '</%s>' % n
    """
