#! /usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import getopt, sys
from pyOGSI.stub import *

verbose = 1
obj_paths = {
    'counter':'/ogsa/services/samples/counter/basic/CounterFactoryService',
    'google':'/ogsa/services/samples/google/GoogleSearchFactoryService',
    'weather':'/ogsa/services/samples/weather/WeatherFactoryService'  
    }

def doCommandLine():
    try:
        (opts, args) = getopt.getopt(sys.argv[1:], 's:p:o:hl',
                                     ('server=', 'port=','obj=','help'))
    except getopt.GetoptError, e:
        print >>sys.stderr, sys.argv[0] + ': ' + str(e)
        sys.exit(-1)
        
    for opt, val in opts:
        if opt in [ '-h', '--help' ]:
            sline = '-'*50
            print '%s\n%s%s\n%s' %(sline,(' '*15),'pyOGSI client',sline)
            print '''Arguments (*Required to run):
            --server  SERVER  (-s SERVER)   *Name of server host
            --port    PORT    (-p PORT)     *Port server is listening on
            --obj     PATH    (-o PATH)     *Object
            --list            (-l )          list of object paths
            --help            (-h )          display this screen\n'''
            sys.exit(0)
        if   opt in ['-l', '--list']:
            for i in obj_paths.values(): print i
            sys.exit(0)
        elif opt in ['-s', '--server']:
            hostname = val
        elif opt in ['-p', '--port']:
            portnum = int(val)
        elif opt in ['-o', '--obj']:
            obj = val
        else:
            print sys.argv[0] + ': Usage error; try --help.'
            sys.exit(-1)

    if args or len(sys.argv)<7:
        print sys.argv[0] + ': Usage error; try --help.'
        sys.exit(-1)
    else:
        print 'argv: ', sys.argv

    return hostname, portnum, obj

'''
Sample Services:
  work:
     samples/counter/basic/CounterFactoryService
     samples/google/GoogleSearchFactoryService

  todo:
      samples/weather/WeatherFactoryService
      samples/chat/ChatFactoryService

'''
def get_wsdl(inst_path, hostname, portnum):
    url = 'http://' + hostname + ':' + str(portnum)
    if  inst_path[-14:] == 'FactoryService':
        url += '/schema/core/factory/factory_service.wsdl'
    elif inst_path[:inst_path.rfind('/')] == obj_paths['counter']:
        url += '/schema/samples/counter/counter_service.wsdl'
    elif inst_path[:inst_path.rfind('/')] == obj_paths['google']:
        url += '/schema/samples/google/google_search_service.wsdl'
    elif inst_path[:inst_path.rfind('/')] == obj_paths['weather']:
        url += '/schema/samples/weather/weather_service.wsdl'
    else:
        raise RuntimeError, 'cant handle instance'
    return Wsdl(url)

'''
***THE NEW createService ***
  POST  /ogsa/services/samples/counter/basic/CounterFactoryService
  HTTP/1.0\r\n Content-Type:  text/xml;
     charset=utf-8\r\n Accept:  application/soap+xml,  application/dime,
   multipart/related,  text/*\r\n User-Agent :  Axis/1.0\r\nHost:
   131.243.2.109:8082\r\nCache-Control:  no-cache\r\nPragma:
   no-cache\r\nSOAPAction: "http://ogsa.gridforum.org/factory/factory#createService"
   \r\nContent-Length: 605\r\n\r\n<?xml version="1.0" encoding="UTF-8"?>\n
     <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
                       xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                       xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
       <soapenv:Header>
         <ns1:invocationID id="JOSH"
                       xmlns:ns1="http://ogsa.globus.org/"/>
       </soapenv:Header>
       <soapenv:Body>
         <createService xmlns="http://ogsa.gridforum.org/factory/factory">
           <ns2:creation
                xmlns:ns2="http://www.gridforum.org/namespaces/2002/10/gridServices">
             <ns2:serviceParameters xsi:nil="true"/>
           </ns2:creation>
         </createService>
       </soapenv:Body>
     </soapenv:Envelope>

*** so far ***
<SOAP-ENV:Header>
  <ns1:invocationID id="JOSH" xmlns:ns1="http://ogsa.globus.org/"/>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
  <createService>
    <ns1:creation 
         xmlns:ns1="http://ogsa.gridforum.org/types">
    </ns1:creation>
  </createService>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

def main():
    hostname, portnum, inst_path = doCommandLine()
    kw = {'ssl':0,
          'namespace':'ns1',
          'host':hostname,
          'port':portnum,
          'tracefile':sys.stdout,
          'url':(inst_path+' HTTP/1.0\r\n')}

    ws = get_wsdl(inst_path, hostname, portnum)
    stg = StubGenerator()
    stubby = stg.generate_stub(ws)
    
    print "stubby    :  %s" %(stubby)
    print "dir       : ", stubby.op_requests.__str__()
    a = stubby(**kw)
    print "stubby inst: %s" %(str(a))

    #
    # TESTING
    #
    if inst_path[-14:] == 'FactoryService':
        args_dict = a.get_args_dict('createService')
        #for k in args_dict.keys():
        
        #args_dict['serviceParameters'] = 'JOSH'
        print "XXXDEBUG: ", args_dict
        #else:
        a.add_soap_header('<ns1:invocationID id="JOSH" xmlns:ns1="http://ogsa.globus.org/"/>')
        a.createService(**args_dict)
            
    elif inst_path[:inst_path.rfind('/')] == obj_paths['google']:
        args_dict = a.get_args_dict('doGoogleSearch')
        args_dict['q'] = 'epiphone sheraton'
        args_dict['key'] = 'hBG+ed8alnS+Ym2JOS+1SBNrpTft+iWi'
        args_dict['maxResults'] = 5
        args_dict['start'] = 10
        args_dict['filter'] = 0
        args_dict['safeSearch'] = 0
        a.doGoogleSearch(**args_dict)
    elif inst_path[:inst_path.rfind('/')]== obj_paths['counter']:
        args_dict = a.get_args_dict('add')
        args_dict['value'] = 10
        a.add(**args_dict)
        a.subtract(**args_dict)
        a.getValue()
    elif inst_path[:inst_path.rfind('/')]== obj_paths['weather']:
        args_dict = a.get_args_dict('setZipCode')
        args_dict['zipCode'] = 94610
        try:
            a.setZipCode(**args_dict)
        except Exception,ex:
            print 'DEBUG NOT HANDLING xsd:float response'    
        a.getTemp()
    else:
        raise RuntimeError, 'service not supported'        
    return 

if __name__ == '__main__':
    main()
